#!/bin/sh
# jehsom's and dn's glftpd installer script
  VERSION="1.2.1"

# REPORT ANY BUGS TO dn (#glftpd@efnet)

# CHANGELOG
#
# v1.2.1:  Changed the text for TLS certificate creation since its no longer
#          required for the installation.
#
#          Copy egrep to the rest of the binaries as well.
#
#          Added www.glftpd.at to recomended sites =)
#------------------------------------------------------------------------------
# v1.2:
# new:     Included installation support for TLS / Turranius.
#
# v1.14:
# fix:     There was a bug in adding the proper -s string for glstrings if
#          you didn't use the default install dir of /glftpd.  This has now
#          been fixed.
# v1.13:
# fix:     there was the possibity of your xinetd.conf file getting erased.
#          Code was changed to first verify if the /etc/xinetd.d directory exists,
#          if not, it will be created and the includedir line added to your
#          xinetd.conf
# v1.12
# change:  updated a few warning messages.
# new:     you will now be told if the private group you are trying to use
#          already exists and options to change it.
# change:  if your system contains both inetd and xinetd config files, you
#          will be asked what you want to use.  If only one can be found, then
#          that will be used automatically.
# v1.11
# fix:     changed the code when restarting xinetd, it was incompatible with
#          some systems.  
# change:  will now look at /etc/xinetd.conf and verify you have set the
#          includedir for xinetd.d, if so a separate glftpd file will
#          be created, else the information will be added to /etc/xinetd.conf
# v1.10
# change:  added some more binaries to the required binaries list, mainly
#          to make Bloody_A happy.
# fix:     script will now look for a compiler to user before compiling
#          the sources, you will get an error message if one can not be 
#          found.
# v1.09
# fix:     some more bugs in the code.
# v1.08
# fix:     some of my own grammar mistakes.
# change:  update a lot of the code to improve things, thanks to our good
#          friend, Jehsom.
# v1.07 -  dn
# new:     will now ask if you want to run glftpd in a jailed environment and
#          set it all up accordingly.
# change:  the output when copying the needed binaries or compiling the source
#          files.  Now it just tells you which have failed, or if all were 
#          successful
# fix:     Now the glroot path and the jaildir path must start with a /
# change:  Instead of exiting if the glroot path or jaildir path is a /,
#          you will now get to enter a new dir name
# fix:     Crontab entries will be removed on a reinstall so you don't get 
#          duplicates
# new/fix: will now configure glftpd properly for use with or without tcpd
# fix:     there was a bug in the [T]ry Again option when creating the install
#          dir for glftpd
# change:  updated the final message to include jehsom's new website
#          information and added mine. :)
# fix:     some spelling mistakes (has Archimede been in here?)
# change:  removed -r /etc/glftpd.conf and -n 1 from the inetd and xinetd lines as
#          they are they default.
# change:  remove -r /etc/glftpd.conf from the reset line in crontab as that is the
#          default
# v1.06 -  maestro^
# new:     permissions set on ftp-data/users
# change:  glftpd.conf now moved to glftpd.conf.dist after install
# fix:     not properly reinstalling on top of itself from the glroot dir
# fix:     some error messages werent printing properly

#DO NOT TOUCH ANYTHING BELOW THIS LINE
#--------------------------------------------------------------------------------------

# Run the script with bash
[ "$foundbash" != "1" ] && {
    export foundbash="1"
    exec bash -x "$0" "$@" 2>./installgl.debug
}

# Set null glob options
allow_null_glob_expansion=1
shopt -s nullglob

# Be sure we're root
[ "$UID" = "0" ] || {
    echo "You must be root to execute to install glftpd."
    exit 1
}

# Make sure we're in the right place
cd "$(dirname $0)"
[ -f "bin/glftpd" ] || {
    echo "Please run $(basename $0) from the glftpd installation dir."
    exit 1
}

# Set system type
case $(uname -s) in
    Linux)
        os=linux
        ;;
    *[bB][sS][dD]*)
        os=bsd
        ;;
    *)
        echo "Sorry, but this installer does not support the $(uname -s) platform."
        echo "You will have to install glftpd manually."
        exit 1
        ;;
esac

# Important bins to have in the glftpd bin dir
BINS="sh cat grep egrep unzip wc find ldconfig ls bash mkdir rmdir rm mv cp awk ln basename dirname head tail cut tr wc sed date sleep touch gzip zip"

# Ensure we have all useful paths in our $PATH
PATH="$PATH:/bin:/usr/bin:/usr/local/bin:/sbin:/usr/sbin:/usr/local/sbin:\
/usr/libexec:/usr/compat/linux/bin:/usr/compat/linux/sbin:/usr/compat/linux/usr/bin:\
/usr/compat/linux/usr/sbin"

echo "Welcome to jehsom's glFtpD installer (raped by Turranius), version $VERSION"
echo ""

echo "Do you want to use tcpd? If you are not sure then you should"
echo "use it.  If you decided to change this at a later time, please"
echo "search for tcpd in the glftpd.docs for the required changes."
echo ""

until [ -n "$usetcpd" ]; do
    echo -n "Use tcpd? [Y]es [N]o: "
    read usetcpd
    case $usetcpd in
        [Nn])
            echo "Installing glftpd without tcpd."
            usetcpd="n"
        ;;
        [Yy])
            tcpd="$(which tcpd | grep "^/")"
            [ -z "$tcpd" ] && {
                echo ""
                echo "It seems you don't have tcpd installed, install it and try"
                echo "again, or select not to use it next time.  Aborting."
                exit 1
            }        
            echo "Installing glftpd with tcpd."
            usetcpd="y"
        ;;
        *)
            unset usetcpd
            continue
        ;;
    esac
done

echo ""
echo "Do you want to run glftpd in a \"Jailed\" environment?  In this"
echo "environment a private directory will be created and glftpd will"
echo "be installed inside.  Regular shell users will not be able to get"
echo "inside this private directory.  The glftpd.conf is also moved"
echo "inside for added security and a new group will be created so"
echo "you and other users you specify can access glftpd through the shell"
echo ""

until [ -n "$jail" ]; do
    echo -n "Use a jailed environment? [Y]es [N]o: "
    read jail
    case $jail in
        [Nn])
            jail="n"
        ;;
        [Yy])
            jail="y"
            echo ""
            echo "Creating the jailed environment."
            echo ""
            until [ -n "$jaildir" ]; do
                echo -n "Please enter the private directory to install glftpd inside [/jail]: "
                read jaildir
                case $jaildir in
                    /)
                        echo "You can't have / as your private dir!  Try again."
                        echo ""
                        unset jaildir
                        continue
                    ;;
                    /*|"")
                        [ -z "$jaildir" ] && jaildir="/jail"
                        [ -d "$jaildir" ] && { 
                            echo -n "Path already exists. [D]elete it, [A]bort, [T]ry again, [I]gnore? "
                            read reply
                            case $reply in
                                [dD]*) rm -rf "$jaildir" ;;
                                [tT]*) unset jaildir; continue ;;
                                [iI]*) ;;
                                *) echo "Aborting."; exit 1 ;;
                            esac
                        }
                        mkdir -p "$jaildir"
                        continue
                    ;;
                    *)
                        echo "The private directory must start with a \"/\".  Try again."
                        echo ""
                        unset jaildir
                        continue
                    ;;
                esac
            done
        ;;
    esac
done

[ "$jail" = "y" ] && {
    echo ""
    echo "Do you want to create a private group?  If you say no then only root will"
    echo "be able to access glftpd.  Otherwise you can add other shell users to the"
    echo "group so they can access glftpd from the shell."
    echo ""
    
    until [ -n "$useprivgroup" ]; do
        echo -n "Use a private group? [Y]es [No]: "
        read useprivgroup
        case $useprivgroup in
            [Nn])
                useprivgroup="n"
                echo -n "Setting permissions on $jaildir . . . "
                chmod 700 "$jaildir"
                echo "Done."
                echo ""
            ;;
            [Yy])
                useprivgroup="y"
                case $os in
                    linux)
                        groupcmd="group"
                    ;;
                    bsd)
                        groupcmd="pw group"
                    ;;
                esac
                until [ -n "$privgroup" ]; do
                    echo -n "What would you like your private group to be called? [glftpd]: "
                    read privgroup
                    [ -z "$privgroup" ] && privgroup="glftpd"
                    makegroup="1"
                    [ -n "$(grep $privgroup /etc/group)" ] && {
                        groupusers="$(grep ^glftpd /etc/group | cut -d ':' -f4)"
                        [ -z "$groupusers" ] && groupusers="No users previously added"
                        echo -e '\033[1;31mWarning\033[0m:' Group already exists with the following users: $groupusers 
                        echo ""
                        echo -n "What would you like to do: [D]elete it, [A]bort, [T]ry again, [I]gnore? "
                        read reply
                        case $reply in
                            [dD]*) "$groupcmd"del $privgroup; ;;
                            [tT]*) unset privgroup; continue ;;
                            [iI]*) makegroup="0";;
                             *) echo "Aborting."; exit 1 ;;
                        esac
                    }
                done 
                
                if [ "$makegroup" -eq "1" ]; then
                    echo -n "Creating private group . . . "
                    "$groupcmd"add $privgroup
                
                    if [ $? -eq 0 ]; then
                        echo "Done."
                        echo ""
                        echo -n "Who should have access to glftpd? (separate with ,): "
                        read jailusers
                        if [ -z "$jailusers" ]; then
                            echo -e '\033[1;31mWarning\033[0m:' No users added, only root can access glftpd.
                        else
                            oIFS=$IFS
                            IFS=,
                            for user in $jailusers; do
                            grep "^$user:" /etc/passwd > /dev/null || {
                                echo -e '\033[1;31mWarning\033[0m:' user $user does not exist.  Adding anyway.
                            }
                            done
                            IFS=$oIFS
                            sed "s/^$privgroup:.*/&$jailusers/" /etc/group > /etc/group.new
                            mv -f /etc/group.new /etc/group
                        fi
                        echo -n "Setting permissions on $jaildir . . . "
                        chmod 750 "$jaildir"
                        chown root."$privgroup" "$jaildir"
                        echo "Done."
                        echo ""
                    else
                        echo -e '\033[1;31mFailed!\033[0m'
                        echo ""
                        echo "The private group could not be created."
                        echo "Please create this group manually and add yourself and any other shell"
                        echo "users to the group who should have access to glftpd.  You will also"
                        echo "need to chown root.$privgroup $jaildir and chmod 750 $jaildir to set the"
                        echo "proper permissions.  If this is not done, only root can access glftpd."
                        echo ""
                        echo -n "Setting permissions on $jaildir . . . "
                        chmod 700 "$jaildir"
                        echo "Done."
                        echo ""
                    fi
                else
                    echo "Using group \"$privgroup\".  The following users have access: $groupusers"
                    echo -n "Setting Permissions on $jaildir . . . "
                    chmod 750 "$jaildir"
                    chown root."$privgroup" "$jaildir"
                    echo "Done."
                    echo ""
                fi     
            ;;
            *)
                unset useprivgroup
                continue
            ;;
        esac
    done
}

# Get a valid installation path
until [ -n "$glroot" ]; do
    if [ "$jail" = "y" ]; then
        echo -n "Please enter the directory inside $jaildir to install glftpd to [/glftpd]: "
    else
        echo -n "Please enter the directory to install glftpd to [/glftpd]: "
    fi
    read glroot
    case $glroot in
        /)
            echo "You can't have / as your private dir!  Try again."
            echo ""
            unset glroot
            continue
        ;;
        /*|"")
            [ -z "$glroot" ] && glroot="/glftpd"
            [ -d "$jaildir$glroot" ] && {
                echo -n "Path already exists. [D]elete it, [A]bort, [T]ry again, [I]gnore? "
                read reply
                case $reply in
                    [dD]*) rm -rf "$jaildir$glroot" ;;
                    [tT]*) unset glroot; continue ;;
                    [iI]*) ;;
                    *) echo "Aborting."; exit 1 ;;
                esac
            }
            mkdir -p "$jaildir$glroot" || unset glroot; continue
        ;;
        *)
            echo "The private directory must start with a \"/\".  Try again."
            echo ""
            unset glroot
            continue
        ;;
    esac
done

echo -n "Copying glftpd files to $jaildir$glroot . . . "

cp -Rf * .[^.]* "$jaildir$glroot"
echo "Done."

if [ "$jail" = "y" ]; then
    confdir="$jaildir"
else
    confdir="/etc"
fi
echo -n "Copying glftpd.conf to $confdir . . . "

if [ -f glftpd.conf ]; then
    conffile="glftpd.conf"
elif [ -f glftpd.conf.dist ]; then
    conffile="glftpd.conf.dist"
else
    echo "FAILED!"
    echo "You must find glftpd.conf and copy it to $confdir manually"
    echo "and change the rootpath to $confdir$glroot"
fi 

[ -n "$conffile" ] && {
    [ -f "$confdir/glftpd.conf" ] && {
        echo -n "Backup up old glftpd.conf . . . "
        mv "$confdir/glftpd.conf" "$confdir/glftpd.conf.bak"
    }
    sed "s:^\([ 	]*rootpath[ 	]*\).*$:\1$jaildir$glroot:" $conffile > "$confdir/glftpd.conf" 
    [ -f glftpd.conf ] && mv glftpd.conf glftpd.conf.dist
    echo "Done."
}

echo -e "\nCopying required binaries to $jaildir$glroot/bin:"
for bin in $BINS; do
    type $bin > /dev/null || {
        failed="$failed   Failed to copy: \033[1;31m$bin\033[0m\n"
        continue    
    }
    cp -f "$(which $bin)" "$jaildir$glroot/bin"
done

if [ -n "$failed" ]; then
    echo -e "$failed"
    echo "Please find and copy these manually to $jaildir$glroot/bin"
else
    echo "All binaries successfully copied."
fi

echo -ne "\nMaking glftpd's /dev/null & /dev/zero . . . "
case $os in
    linux)
        mknod -m666 "$jaildir$glroot/dev/null" c 1 3
        mknod -m666 "$jaildir$glroot/dev/zero" c 1 5
        ;;
    bsd)
        mknod "$jaildir$glroot/dev/null" c 2 2
        mknod "$jaildir$glroot/dev/zero" c 2 12
        chmod 666 "$jaildir$glroot/dev/"*
        ;;
esac
echo "Done."
echo ""

echo "Compiling source files in $jaildir$glroot/bin/sources to $jaildir$glroot/bin:" 
echo "(NOTE: Compiling can take a couple of minutes, please be patient.)"
if [ -n "$(which gcc)" ]; then
    usegcc="$(which gcc)"
elif [ -n "$(which cc)" ]; then
    usegcc="$(which cc)"
else
   echo "Error: Can't find a compiler to use.  Please install one and compile all"
   echo "       the scripts in $jaildir$glroot/bin/sources and place them in"
   echo "       $jaildir/$glroot/bin.  Note these files are not necessary for"
   echo "       glftpd to operate, they are extra external scripts."
fi
[ -n "$usegcc" ] && {
    for cfile in $jaildir$glroot/bin/sources/*.c; do
        base="$(basename "${cfile%.c}")"
        [ -f "$jaildir$glroot/bin/$base" ] && rm -f "$jaildir$glroot/bin/$base"
        gcc -o "$jaildir$glroot/bin/$base" "$cfile" > /dev/null 2>&1 || \
        failedc="$failedc   Failed to compile: \033[1;31m$cfile\033[0m\n" 
    done
    if [ -z "$failedc" ]; then
        echo "All source files successfully compiled."
    else
        echo -e "$failedc"
    fi
}

echo -e "\nCopying required shared library files:"
ldd $jaildir$glroot/bin/* | grep "=>" | sed 's:^.* => \(/[^ ]*\).*$:\1:' | 
sort | uniq | while read lib; do
    echo -n "   $(basename $lib): "
    if [ -f "$lib" ]; then
        cp -f "$lib" "$jaildir$glroot/lib"
        echo "OK"
    elif [ -f "/usr/compat/linux/$lib" ]; then
        cp -f "/usr/compat/linux/$lib" "$jaildir$glroot/lib"
        echo "OK"
    else
        echo -e '\033[1;31mFailed!\033[0m'" You must find & copy $(basename $lib) to $jaildir$glroot/lib manually."
    fi
done
case $os in bsd)
    echo -e "\nCopying ELF interpreter and linux libraries:"
    echo "(NOTE: Searches can take a couple of minutes, please be patient.)"
    bsdlibs="/usr/libexec/ld-elf.so.1 /usr/compat/linux/lib/ld-linux.so.1"
    mkdir -p "$jaildir$glroot/usr/libexec"
    for bsdlib in $bsdlibs; do
        echo -n "   $(basename $bsdlib): "
        if [ -e "$bsdlib" ]; then
            cp -f "$bsdlib" "$jaildir$glroot/usr/libexec"
            echo "OK"
        else
          echo -n "Searching . . . " 
           file=$(find / -name $(basename $bsdlib) | head -1)
           if [ -n "$file" ]; then
               cp -f "$file" "$jaildir/$glroot/usr/libexec"
               echo "OK"
           else
               echo -e '\033[1;31mFailed!\033[0m'
               libfailed="1"
           fi
        fi
    done
    [ $libfailed -eq 1 ] && echo "You must install and copy the missing libraries to $jaildir$glroot/usr/libexec manually." 
esac   

echo -ne "\nConfiguring the shared library cache . . . "
case $os in
    linux)
        echo "/lib" > "$jaildir$glroot/etc/ld.so.conf"
        chroot "$jaildir$glroot" /bin/ldconfig
        ;;
    bsd)
        mkdir -p "$jaildir$glroot/usr/lib"
        mkdir -p "$jaildir$glroot/var/run"
        chroot "$jaildir$glroot" /bin/ldconfig -elf /lib
        chroot "$jaildir$glroot" /bin/ldconfig -aout /lib
        chroot "$jaildir$glroot" /bin/ldconfig
        echo "/lib" > "$jaildir$glroot/etc/ld.so.conf"
        chroot "$jaildir$glroot" /bin/ldconfig
        ;;
esac
echo "Done."

until echo "$port" | grep -E "^[0-9]+$" > /dev/null && [ "$port" -lt 65535 ]; do
    echo -en "\nEnter the port you would like glftpd to listen on [21]: "
    read port
    [ -z "$port" ] && port=21
done

echo -ne "Setting userfile permissions . . . "
chmod 755 "$jaildir$glroot/ftp-data/users"
chmod 644 "$jaildir$glroot/ftp-data/users/"*
echo "Done."

echo -n "Adding glftpd service to /etc/services . . . "

{ grep -v ^glftpd /etc/services;
  echo "glftpd   $port/tcp"
} > /etc/services.new
mv -f /etc/services.new /etc/services
echo "Done."
echo ""

# Configure inetd or xinetd as appropriate
[ "$jail" = "y" ] && confpath="-r $jaildir/glftpd.conf"
[ "$usetcpd" = "n" ] && dummyflag="glftpd"

if [ -f /etc/inetd.conf -a -f /etc/xinetd.conf ]; then
    until [ -n "$whichnetd" ]; do
        echo -n "Do you want to use [I]netd or [X]inetd? "
        read whichnetd
        case $whichnetd in
            [xX]) netd="x" ;;
            [iI]) netd="i" ;;
            *) unset whichnetd; continue ;;
        esac
    done
elif [ -f /etc/inetd.conf ]; then
    netd="i"
elif [ -f /etc/xinetd.conf ]; then
    netd="x"
else
    echo "ERROR: Can't determine if you are using inetd or xinetd!"
    echo "Please fix this problem and re-run the installation script"
    exit 1
fi

## Setup path to glstrings.bin if we're in a jailed enviroment.
if [ -n "$jaildir" ]; then
    glstrings="-s$jaildir$glroot/bin/glstrings.bin"
elif [ "$glroot" != "/glftpd" ]; then
    glstrings="-s$glroot/bin/glstrings.bin"
fi

### TLS INSTALLATION / 2003-09-01 - Turranius
### Mod 2004-01-04 - Fixed help text for 2.0+ / Turranius

proc_createcert() {
  echo " "
  echo "Please specify a generic name for this certificate."
  echo "This can be any name but should say something about the ftp server"
  echo -n "like the name for it perhaps? : "

  until [ -n "$tlsname" ]; do
    read tlsname
    if [ -z "$tlsname" ]; then
      echo " "
      echo -n "You must select a name : "
      unset tlsname; continue
    else
      continue
    fi
  done

  echo " "
  echo "Please wait while creating certificate. This WILL take quite some time depending on machine power."
  echo "Using servbase: $tlsname  Using openssl: $opensslbin"
  echo "Dont worry, your computer has not frozen..."
  echo " "

  type=dsa
  servbase=$tlsname
  base=ftpd-dsa
  $opensslbin rand -out gunk 1024
  $opensslbin dsaparam -rand gunk -out $base.dsaparam 1024
  $opensslbin dhparam -rand gunk -out $base.dh 1024
  option=$base.dsaparam
  rm -f gunk	

  $opensslbin gen$type -out $base.key $option
  $opensslbin req -new -x509 -days 365 -key $base.key -out $base.crt 2>/dev/null << EOF
.
. 
.
.
.
$servbase
.
EOF

  if [ ! -e "$base.key" ]; then
    echo "Error: Could not create $base.key. You need to read README.TLS and do thia manually."
    TLSERROR=TRUE
  else
    cat $base.key > $base.pem
  fi

  if [ "$TLSERROR" != "TRUE" ]; then
    if [ ! -e "$base.crt" ]; then
      echo "Error: Could not create $base.crt. You need to read README.TLS and do thia manually."
      TLSERROR=TRUE
      rm -f $base.pem
    else
      cat $base.crt >> $base.pem
    fi
  fi

  if [ "$TLSERROR" != "TRUE" ]; then
    if [ ! -e "$base.dh" ]; then
      echo "Error: Could not create $base.dh. You need to read README.TLS and do thia manually."
      TLSERROR=TRUE
      rm -f $base.pem
    else
      cat $base.dh >> $base.pem
    fi
  fi
  rm -f $base.key $base.crt $base.dh $base.dsaparam

  if [ -e "$base.pem" ]; then
    mv -f $base.pem $tls/
    echo "Moving $base.pem to $tls . . . Done"
    tlsfull="-z cert=$tls/$base.pem"
  else
    if [ "$TLSERROR" != "TRUE" ]; then
      echo "Error. Could not copy $base.pem to $tls. Have a look at README.TLS and set this up manually."
    fi
  fi
}

opensslbin=`which openssl 2>/dev/null`
if [ -z "$opensslbin" ]; then

  echo "It would appear that you do not have openssl installed."
  echo "This installation will continue, but TLS support will not be installed."
  echo "You can install openssl and after that, check README.TLS for how"
  echo "to create the certificate and change glftpd.conf"
  echo "This is, however, not required for glftpd to operate."
  echo " " 

else

  echo "We will now setup the TLS system. Since v1.30, TLS is built into glftpd but"
  echo "it is no longer required with v2.0 and up."
  echo " "
  echo "Do you wish to create a certificate now? It will take some time."
  echo "If you have no idea what this is, select No."
  echo " "
  echo -n "Create ftpd-dsa.pem now? [Y/N]: "
  until [ -n "$tlsq" ]; do
    read tlsq
    case $tlsq in
      [yY]) continue ;;
      [nN]) tls="NOTNOW"; continue ;;
      *) unset tlsq; echo -n "[Y]es or [N]o : "; continue ;;
    esac
  done
     
  if [ "$tls" != "NOTNOW" ]; then
    until [ -n "$tls" ]; do

        if [ "$jail" = "y" ]; then
            echo " "
            echo -n "Please specify location, inside $jaildir$glroot, to install ftpd-dsa.pem to [/etc]: "
        else
            echo " "
            echo -n "Please specify location, inside $glroot, to install ftpd-dsa.pem to [/etc]: "
        fi
        read tls
        case $tls in
            /)
                echo "ftpd-dsa.pem should not reside in / - Try again."
                echo ""
                unset tls
                continue
            ;;
            /*)
                if [ ! -d "$jaildir$glroot/$tls" ]; then
                  echo " "
                  echo "$jaildir$glroot/$tls does not exist. Try again."
                  unset tls
                  continue
                else
                  tls="$jaildir$glroot/$tls"
                fi
            ;;
            *) 
                tls="$jaildir$glroot/etc"
            ;;
         esac
    done

    ## Jump to cert creation phase.
    proc_createcert
  fi
fi
unset opensslbin

### END TLS INSTALLATION / 2003-09-01 - Turranius

if [ "$netd" = "i" ]; then
    echo -n "Configuring inetd for glftpd . . . "
    { grep -v ^glftpd /etc/inetd.conf
        echo "glftpd stream tcp nowait root $tcpd $jaildir$glroot/bin/glftpd $dummyflag -l -o $tlsfull -i $confpath \
        $glstrings" | tr -s ' ' 
    } > /etc/inetd.conf.new
    mv -f /etc/inetd.conf.new /etc/inetd.conf
    echo "Done."
    echo -n "Restarting inetd . . . "
    if killall -HUP inetd; then
        echo "Success."
    else
        echo -e '\033[1;31mFailed!\033[0m You must restart inetd before using glftpd.'
    fi
else
    echo -n "Configuring xinetd for glftpd . . . "
    [ ! -d "/etc/xinetd.d" ] && {
        mkdir /etc/xinetd.d
        echo "includedir /etc/xinetd.d" >> /etc/xinetd.conf
    } 
    cat <<EOF > /etc/xinetd.d/glftpd
service glftpd
{
    disable = no
    flags           = REUSE ${tcpd:+NAMEINARGS}
    socket_type     = stream
    protocol        = tcp
    wait            = no
    user            = root
    server          = ${tcpd:-$jaildir$glroot/bin/glftpd}
    server_args     = ${tcpd:+$jaildir$glroot/bin/glftpd} -l -i -o $confpath $glstrings $tlsfull 
}
EOF
    echo "Done."
    echo -n "Restarting xinetd . . . "
    if killall -USR2 xinetd; then
        echo "Success."
        echo ""
        echo -e '\033[1;31mIf your system is using RH 7.3, you MUST restart xinetd MANUALLY!'
    else
        echo -e '\033[1;31mFailed!\033[0m You must start xinetd before using glftpd.'
    fi
fi

echo -ne "\nAdding crontab entry to tabulate site stats nightly . . . "

{ crontab -l | grep -v "bin/reset"
  echo "0  0 * * *      $jaildir$glroot/bin/reset $confpath"
} | crontab - > /dev/null

echo "Done."

echo ""
echo "Congratulations, glFtpD has been installed. Scroll up and note any errors"
echo "that need fixing. A log of the installation script is in ./installgl.debug"
echo ""

if [ "$tlsfull" ]; then
  echo "-> IMPORTANT !!!!"
  echo "-> If you get TLS errors of any kind, read instructions in README.TLS included"
  echo "-> in this package!"
  echo ""
fi

echo "To get your site running, you must edit /etc/glftpd.conf according to the"
echo "instructions in $glroot/glftpd.docs. For help, visit #glftpd on EFnet after"
echo "you've read through them a few time."
echo ""
echo "After configuring glftpd, visit the following websites for additional"
echo "scripts to give your site some style!:"
echo "	Turranius - http://www.grandis.nu/glftpd"
echo "	Jehsoms - http://runslinux.net/"
echo "	dn's and ip's - http://www.chimera-coding.com"
echo "	D-ViBE's collection - http://www.glftpd.at"
echo ""
echo "The official glftpd homepage is located at http://www.glftpd.com"
echo ""
echo "                                 Thanks for your support!"
echo "                                 dn and the glFtpD team"
echo
exit 0
